-- Test home
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(30);

set search_path to camper, public;

select has_table('home');
select has_pk('home');
select table_privs_are('home', 'guest', array['SELECT']);
select table_privs_are('home', 'employee', array['SELECT']);
select table_privs_are('home', 'admin', array['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('home', 'authenticator', array[]::text[]);

select has_column('home', 'company_id');
select col_is_pk('home', 'company_id');
select col_is_fk('home', 'company_id');
select fk_ok('home', 'company_id', 'company', 'company_id');
select col_type_is('home', 'company_id', 'integer');
select col_not_null('home', 'company_id');
select col_hasnt_default('home', 'company_id');

select has_column('home', 'slogan');
select col_type_is('home', 'slogan', 'text');
select col_not_null('home', 'slogan');
select col_hasnt_default('home', 'slogan');


set client_min_messages to warning;
truncate home cascade;
truncate company_host cascade;
truncate company_user cascade;
truncate company cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, rtc_number, tourist_tax, tourist_tax_max_days, country_code, currency_code, default_lang_tag)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', '', 60, 7, 'ES', 'EUR', 'ca')
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', '', 60, 7, 'FR', 'USD', 'ca')
     , (6, 'Company 5', 'XX345', '', '777-777-777', 'c@c', '', '', '', '', '', '', 60, 7, 'FR', 'USD', 'ca')
;

insert into company_user (company_id, user_id, role)
values (2, 1, 'admin')
     , (4, 5, 'admin')
;

insert into company_host (company_id, host)
values (2, 'co2')
     , (4, 'co4')
;

insert into home (company_id, slogan)
values (2, 'Slogan 2')
     , (4, 'Slogan 4')
;

prepare home_data as
select company_id, slogan
from home
order by company_id, slogan;

set role guest;
select bag_eq(
	'home_data',
	$$ values (2, 'Slogan 2')
	        , (4, 'Slogan 4')
	$$,
	'Everyone should be able to list all home texts across all companies'
);
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog', 'co2');

select lives_ok(
	$$ delete from home where company_id = 2 $$,
	'Admin from company 2 should be able to delete home texts from that company.'
);

select bag_eq(
	'home_data',
	$$ values (4, 'Slogan 4')
	$$,
	'The row should have been deleted.'
);

select lives_ok(
	$$ insert into home(company_id, slogan) values (2, 'Another Slogan') $$,
	'Admin from company 2 should be able to insert a new home texts to that company.'
);

select bag_eq(
	'home_data',
	$$ values (2, 'Another Slogan')
	        , (4, 'Slogan 4')
	$$,
	'The new row should have been added'
);

select lives_ok(
	$$ update home set slogan = 'Slogan 2' where company_id = 2 $$,
	'Admin from company 2 should be able to update home texts of that company.'
);

select bag_eq(
	'home_data',
	$$ values (2, 'Slogan 2')
	        , (4, 'Slogan 4')
	$$,
	'The row should have been updated.'
);

select throws_ok(
	$$ insert into home (company_id, slogan) values (6, 'Slogan 6') $$,
	'42501', 'new row violates row-level security policy for table "home"',
	'Admin from company 2 should NOT be able to insert new home texts to company 6.'
);

select lives_ok(
	$$ update home set slogan = 'Nope' where company_id = 4 $$,
	'Admin from company 2 should not be able to update new home texts of company 4, but no error if company_id is not changed.'
);

select bag_eq(
	'home_data',
	$$ values (2, 'Slogan 2')
	        , (4, 'Slogan 4')
	$$,
	'No row should have been changed.'
);

select throws_ok(
	$$ update home set company_id = 6 where company_id = 2 $$,
	'42501', 'new row violates row-level security policy for table "home"',
	'Admin from company 2 should NOT be able to move home texts to company 6'
);

select lives_ok(
	$$ delete from home where company_id = 4 $$,
	'Admin from company 2 should NOT be able to delete home texts from company 4, but not error is thrown'
);

select bag_eq(
	'home_data',
	$$ values (2, 'Slogan 2')
	        , (4, 'Slogan 4')
	$$,
	'No row should have been changed'
);

reset role;


select *
from finish();

rollback;

