-- Test tax
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(36);

set search_path to camper, auth, public;

select has_table('tax');
select has_pk('tax' );
select table_privs_are('tax', 'guest', array []::text[]);
select table_privs_are('tax', 'employee', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('tax', 'admin', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('tax', 'authenticator', array []::text[]);

select has_column('tax', 'tax_id');
select col_is_pk('tax', 'tax_id');
select col_type_is('tax', 'tax_id', 'integer');
select col_not_null('tax', 'tax_id');
select col_hasnt_default('tax', 'tax_id');

select has_column('tax', 'company_id');
select col_is_fk('tax', 'company_id');
select fk_ok('tax', 'company_id', 'company', 'company_id');
select col_type_is('tax', 'company_id', 'integer');
select col_not_null('tax', 'company_id');
select col_hasnt_default('tax', 'company_id');

select has_column('tax', 'tax_class_id');
select col_is_fk('tax', 'tax_class_id');
select fk_ok('tax', 'tax_class_id', 'tax_class', 'tax_class_id');
select col_type_is('tax', 'tax_class_id', 'integer');
select col_not_null('tax', 'tax_class_id');
select col_hasnt_default('tax', 'tax_class_id');

select has_column('tax', 'name');
select col_type_is('tax', 'name', 'text');
select col_not_null('tax', 'name');
select col_hasnt_default('tax', 'name');

select has_column('tax', 'rate');
select col_type_is('tax', 'rate', 'tax_rate');
select col_not_null('tax', 'rate');
select col_hasnt_default('tax', 'rate');


set client_min_messages to warning;
truncate tax cascade;
truncate tax_class cascade;
truncate company_host cascade;
truncate company_user cascade;
truncate company cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, rtc_number, tourist_tax, tourist_tax_max_days, country_code, currency_code, default_lang_tag)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', '', 60, 7, 'ES', 'EUR', 'ca')
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', '', 60, 7, 'FR', 'USD', 'ca')
;

insert into company_user (company_id, user_id, role)
values (2, 1, 'admin')
     , (4, 5, 'admin')
;

insert into company_host (company_id, host)
values (2, 'co2')
     , (4, 'co4')
;

insert into tax_class(tax_class_id, company_id, name)
values (5, 2, 'vat')
     , (6, 2, 'irpf')
     , (7, 4, 'vat')
;

insert into tax (company_id, tax_class_id, name, rate)
values (2, 5, 'VAT 21 %', 0.21)
     , (2, 6, 'IRPF -15 %', -0.15)
     , (4, 7, 'VAT 21 %', 0.21)
     , (4, 7, 'VAT 10 %', 0.10)
     , (4, 7, 'VAT 5 %', 0.05)
     , (4, 7, 'VAT 4 %', 0.04)
     , (4, 7, 'VAT 0 %', 0.00)
;

prepare tax_data as
select company_id, name, rate
from tax
order by company_id, rate;

set role employee;
select is_empty('tax_data', 'Should show no data when cookie is not set yet');
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog', 'co2');
select bag_eq(
	'tax_data',
	$$ values ( 2, 'IRPF -15 %', -0.15::tax_rate )
	        , ( 2, 'VAT 21 %', 0.21::tax_rate )
	$$,
	'Should only list taxes of the companies where demo@tandem.blog is user of'
);
reset role;

select set_cookie('12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524/admin@tandem.blog', 'co4');
select bag_eq(
	'tax_data',
	$$ values (4, 'VAT 0 %', 0.00::tax_rate)
	        , (4, 'VAT 4 %', 0.04::tax_rate)
	        , (4, 'VAT 5 %', 0.05::tax_rate)
	        , (4, 'VAT 10 %', 0.10::tax_rate)
	        , (4, 'VAT 21 %', 0.21::tax_rate)
	$$,
	'Should only list taxes of the companies where admin@tandem.blog is user of'
);
reset role;

select set_cookie('not-a-cookie', 'co4');
select throws_ok(
	'tax_data',
	'42501', 'permission denied for table tax',
	'Should not allow select to guest users'
);
reset role;

select throws_ok( $$
	insert into tax (company_id, tax_class_id, name, rate)
	values (2, 6, '  ', 0.22)
	$$,
	'23514', 'new row for relation "tax" violates check constraint "name_not_empty"',
	'Should not allow taxs with blank name'
);

select *
from finish();

rollback;

